<?php

class Dap_Cron {

	/*
	Bunch of static methods to handle transactions and other functions for Cron.

	*/

	/*
	actionType = BULKADDCSVTOPRODUCT
	Payload format #PRODUCTID||CSVFILENAME||ISPAID(Y/N)

	*/
	public static function handleBulkAddCsvToProduct() {
		logToFile("(Dap_Cron.handleBulkAddCsvToProduct() - Method Init.");
		try {

			$dap_dbh = Dap_Connection::getConnection();
			$select_sql = "select
						id,
						payload,
						actionKey
					from
						dap_mass_actions
					where
						status in ('NEW', 'E') and
						actionType = 'BULKADDCSVTOPRODUCT'
					";
			$update_sql = "update
							dap_mass_actions
							set
							status = :status,
							comments = :comments
							where
							id = :id
							";
			$insert_sql = "insert into dap_mass_actions
								(actionType, actionKey, payload, status, priority)
							values ('ADDUSERTOPRODUCT', :key, :payload, 'NEW', 4)
							";

			//logToFile($sql,LOG_DEBUG_DAP);
			$select_stmt = $dap_dbh->prepare($select_sql);
			$update_stmt = $dap_dbh->prepare($update_sql);
			$insert_stmt = $dap_dbh->prepare($insert_sql);

			//execute select
			$select_stmt->execute();

			if ($row = $select_stmt->fetch(PDO::FETCH_ASSOC)) {
				//get payload
				$payload = $row['payload'];
				$parentkey = $row['actionKey'];
				//parse payload
				$tokens = explode("||",$payload);
				if(count($tokens) < 3) {
					$update_stmt->bindParam(':id', $row['id'], PDO::PARAM_INT);
					$update_stmt->bindValue(':status', 'E', PDO::PARAM_STR);
					$update_stmt->bindValue(':comments', 'Payload Format Is Not Correct', PDO::PARAM_STR);
					$update_stmt->execute();
					sendMassActionFailedNE("BulkAddCSVToProduct", "Mass action request format is invalid");
					return;
				}
				//lets get values
				$product_id = $tokens[0];
				$csvfile = $tokens[1];
				$ispaid = $tokens[2];
				//see if file exists and read if it does
				logToFile("(Dap_Cron.handleBulkAddCsvToProduct() - Opening CSV File:".$csvfile);
				$handle = fopen($csvfile, "r");
				logToFile("(Dap_Cron.handleBulkAddCsvToProduct() - Opening CSV File:".$handle);
				while (($data = fgetcsv($handle, 1000, ",")) !== FALSE) {
					logToFile("(Dap_Cron.handleBulkAddCsvToProduct() - CSV Row:".$data);
				    	if(count($data) < 2) {
						$update_stmt->bindParam(':id', $row['id'], PDO::PARAM_INT);
						$update_stmt->bindValue(':status', 'E', PDO::PARAM_STR);
						$update_stmt->bindValue(':comments', 'Error in CSV File'.$data, PDO::PARAM_STR);
						$update_stmt->execute();
						sendMassActionFailedNE("BulkAddCSVToProduct", "Error in CSV File");
						return;
					}
					
					logToFile("(Dap_Cron.handleBulkAddCsvToProduct() - data[0]:".$data[0]); //email
					logToFile("(Dap_Cron.handleBulkAddCsvToProduct() - data[1]:".$data[1]); //firstname
					logToFile("(Dap_Cron.handleBulkAddCsvToProduct() - data[2]:".$data[2]); //lastname
					logToFile("(Dap_Cron.handleBulkAddCsvToProduct() - data[3]:".$data[3]); //username
					
					
					$payload2 = $product_id . "||" . $data[0] . "||" . $data[1];
					if(isset($data[2])) {
						$payload2 = $payload2 . "||" . $data[2] . "||" . $ispaid;
					} else {
						$payload2 = $payload2 . "||" . "||" . $ispaid;
					}
					
					if(isset($data[3])) { //username 
						$payload2 = $payload2 . "||" . $data[3];
					} else {
						$payload2 = $payload2 . "||" . "||" . "||";
					}
					
					
					$key = $product_id.":".$data[0].":".$parentkey;
					logToFile("(Dap_Cron.handleBulkAddCsvToProduct() - Insert Payload:".$payload2);
					try {
						$insert_stmt->bindParam(':payload', $payload2, PDO::PARAM_STR);
						$insert_stmt->bindParam(':key', $key, PDO::PARAM_STR);
						$insert_stmt->execute();
					} catch (PDOException $e) {
						$msg =  $e->getMessage();
						sendMassActionFailedNE("BulkAddCSVToProduct", "Error inserting ADDUSERTOPRODUCT");
						echo $msg;
					}
				}
				fclose($handle);

				//COMPLETED TASK SUCCESSFULLY
				$update_stmt->bindParam(':id', $row['id'], PDO::PARAM_INT);
				$update_stmt->bindValue(':status', 'C', PDO::PARAM_STR);
				$update_stmt->bindValue(':comments', 'Completed Successfully', PDO::PARAM_STR);
				$update_stmt->execute();
			}

			return;
		} catch (PDOException $e) {
			logToFile($e->getMessage(),LOG_FATAL_DAP);
			throw $e;
		} catch (Exception $e) {
			logToFile($e->getMessage(),LOG_FATAL_DAP);
			throw $e;
		}
	}

	/*
	actionType=ADDUSERTOPRODUCT
	Payload format
	#PRODUCTID||EMAILID||FIRSTNAME||LASTNAME (optional)||ISPAID


	*/
	public static function handleAddNewUserToProduct() {
		logToFile("(Dap_Cron.handleAddNewUserToProduct() - Method Init.");
		try {

			$dap_dbh = Dap_Connection::getConnection();
			$select_sql = "select
						id,
						payload
					from
						dap_mass_actions
					where
						status in ('NEW', 'E') and
						actionType = 'ADDUSERTOPRODUCT'
					limit 100
					";
			$update_sql = "update
							dap_mass_actions
							set
							status = :status,
							comments = :comments
							where
							id = :id
							";

			//logToFile($sql,LOG_DEBUG_DAP);
			$select_stmt = $dap_dbh->prepare($select_sql);
			$update_stmt = $dap_dbh->prepare($update_sql);

			//execute select
			$select_stmt->execute();

			while ($row = $select_stmt->fetch(PDO::FETCH_ASSOC)) {
				//get payload
				$payload = $row['payload'];
				//parse payload
				$tokens = explode("||",$payload);
				if(count($tokens) < 4) {
					$update_stmt->bindParam(':id', $row['id'], PDO::PARAM_INT);
					$update_stmt->bindValue(':status', 'E', PDO::PARAM_STR);
					$update_stmt->bindValue(':comments', 'Payload Format Is Not Correct', PDO::PARAM_STR);
					$update_stmt->execute();
					sendMassActionFailedNE("AddUserToProduct", "Payload format is not correct");
					continue;
				}
				
				logToFile("(Dap_Cron.handleAddNewUserToProduct() - tokens[0]:".$tokens[0]); //productid
				logToFile("(Dap_Cron.handleAddNewUserToProduct() - tokens[1]:".$tokens[1]); //email
				logToFile("(Dap_Cron.handleAddNewUserToProduct() - tokens[2]:".$tokens[2]); //firstname
				logToFile("(Dap_Cron.handleAddNewUserToProduct() - tokens[3]:".$tokens[3]); //lastname
				logToFile("(Dap_Cron.handleAddNewUserToProduct() - tokens[4]:".$tokens[4]); // ispaid
				logToFile("(Dap_Cron.handleAddNewUserToProduct() - tokens[5]:".$tokens[5]); // username
																																																										
				//call addUserToProduct($email, $firstname, $lastname, $productid, $ispaid='n')
				logToFile("(Dap_Cron.handleAddNewUserToProduct() - Calling addNewUserToProduct"); // username
				Dap_UsersProducts::addNewUserToProduct($tokens[1], $tokens[2], $tokens[3], $tokens[5], $tokens[0], $tokens[4], "A");
				logToFile("(Dap_Cron.handleAddNewUserToProduct() - Finished addNewUserToProduct"); // username
				//COMPLETED TASK SUCCESSFULLY
				$update_stmt->bindParam(':id', $row['id'], PDO::PARAM_INT);
				$update_stmt->bindValue(':status', 'C', PDO::PARAM_STR);
				$update_stmt->bindValue(':comments', 'Completed Successfully', PDO::PARAM_STR);
				$update_stmt->execute();
			}

			return;
		} catch (PDOException $e) {
			logToFile($e->getMessage(),LOG_FATAL_DAP);
			throw $e;
		} catch (Exception $e) {
			logToFile($e->getMessage(),LOG_FATAL_DAP);
			throw $e;
		}
	}

	/*
	actionType=ADDUSERTOPRODUCT
	Payload format
	#PRODUCTID||EMAILID||FIRSTNAME||LASTNAME (optional)||ISPAID


	*/
	public static function handleAddUserToProduct() {
		logToFile("(Dap_Cron.handleAddUserToProduct() - Method Init.");
		try {

			$dap_dbh = Dap_Connection::getConnection();
			$select_sql = "select
						id,
						payload
					from
						dap_mass_actions
					where
						status in ('NEW', 'E') and
						actionType = 'ADDUSERTOPRODUCT'
					limit 100
					";
			$update_sql = "update
							dap_mass_actions
							set
							status = :status,
							comments = :comments
							where
							id = :id
							";

			//logToFile($sql,LOG_DEBUG_DAP);
			$select_stmt = $dap_dbh->prepare($select_sql);
			$update_stmt = $dap_dbh->prepare($update_sql);

			//execute select
			$select_stmt->execute();

			while ($row = $select_stmt->fetch(PDO::FETCH_ASSOC)) {
				//get payload
				$payload = $row['payload'];
				//parse payload
				$tokens = explode("||",$payload);
				if(count($tokens) < 4) {
					$update_stmt->bindParam(':id', $row['id'], PDO::PARAM_INT);
					$update_stmt->bindValue(':status', 'E', PDO::PARAM_STR);
					$update_stmt->bindValue(':comments', 'Payload Format Is Not Correct', PDO::PARAM_STR);
					$update_stmt->execute();
					sendMassActionFailedNE("AddUserToProduct", "Payload format is not correct");
					continue;
				}
				//call addUserToProduct($email, $firstname, $lastname, $productid, $ispaid='n')
				Dap_UsersProducts::addUserToProduct($tokens[1], $tokens[2], $tokens[3], $tokens[0], $tokens[4], "A");
				//COMPLETED TASK SUCCESSFULLY
				$update_stmt->bindParam(':id', $row['id'], PDO::PARAM_INT);
				$update_stmt->bindValue(':status', 'C', PDO::PARAM_STR);
				$update_stmt->bindValue(':comments', 'Completed Successfully', PDO::PARAM_STR);
				$update_stmt->execute();
			}

			return;
		} catch (PDOException $e) {
			logToFile($e->getMessage(),LOG_FATAL_DAP);
			throw $e;
		} catch (Exception $e) {
			logToFile($e->getMessage(),LOG_FATAL_DAP);
			throw $e;
		}
	}


	/*
	actionType = EMAILTOCSV
	Payload format #CSVFILENAME||SUBJECT||BODY||ATTACHMENTS


	*/
	public static function handleEmailToCsv() {
		logToFile("(Dap_Cron.handleEmailToCsv() - Method Init.");
		try {

			$dap_dbh = Dap_Connection::getConnection();
			$select_sql = "select
						id,
						actionKey,
						payload
					from
						dap_mass_actions
					where
						status in ('NEW', 'E') and
						actionType = 'EMAILTOCSV'
					";
			$update_sql = "update
							dap_mass_actions
							set
							status = :status,
							comments = :comments
							where
							id = :id
							";
			$insert_sql = "insert into dap_mass_actions
								(actionType, actionKey, payload, status, priority)
							values ('EMAIL', :key, :payload, 'NEW', 4)
							";

			//logToFile($sql,LOG_DEBUG_DAP);
			$select_stmt = $dap_dbh->prepare($select_sql);
			$update_stmt = $dap_dbh->prepare($update_sql);
			$insert_stmt = $dap_dbh->prepare($insert_sql);

			//execute select
			$select_stmt->execute();

			if ($row = $select_stmt->fetch(PDO::FETCH_ASSOC)) {
				//get payload
				$payload = $row['payload'];
				$parentkey = $row['actionKey'];
				//parse payload
				$tokens = explode("||",$payload);
				if(count($tokens) < 4) {
					$update_stmt->bindParam(':id', $row['id'], PDO::PARAM_INT);
					$update_stmt->bindValue(':status', 'E', PDO::PARAM_STR);
					$update_stmt->bindValue(':comments', 'Payload Format Is Not Correct', PDO::PARAM_STR);
					$update_stmt->execute();
					return;
				}
				//lets get values
				$csvfile = $tokens[0];
				$subject = $tokens[1];
				$body = $tokens[2];
				$attachments = $tokens[3];
				$dae = Dap_Cron::doesAttachmentsExist($attachments);
				if(!$dae) {
					$update_stmt->bindParam(':id', $row['id'], PDO::PARAM_INT);
					$update_stmt->bindValue(':status', 'E', PDO::PARAM_STR);
					$update_stmt->bindValue(':comments', 'One or more attachments does not exist'.$data, PDO::PARAM_STR);
					$update_stmt->execute();
					// TODO : continue or return ?
					return;
				}
				//see if file exists and read if it does
				logToFile("(Dap_Cron.handleEmailToCsv() - Opening CSV File:".$csvfile, LOG_INFO_DAP);
				$handle = fopen($csvfile, "r");
				logToFile("(Dap_Cron.handleEmailToCsv() - Opening CSV File:".$handle);
				while (($data = fgetcsv($handle, 1000, ",")) !== FALSE) {
					logToFile("(Dap_Cron.handleEmailToCsv() - CSV Row:".$data, LOG_INFO_DAP);
				    	if(count($data) < 2) {
						$update_stmt->bindParam(':id', $row['id'], PDO::PARAM_INT);
						$update_stmt->bindValue(':status', 'E', PDO::PARAM_STR);
						$update_stmt->bindValue(':comments', 'Error in CSV File'.$data, PDO::PARAM_STR);
						$update_stmt->execute();
						// TODO : continue or return ?
						return;
					}
					$payload2 = $data[0] . "||" . $data[1] . "||";
					if(isset($data[2])) {
						$payload2 = $payload2 . $data[2];
					}

					$key = $data[0].":".$parentkey;
					$payload2 = $payload2 . "||" . $subject . "||" . $body . "||" . $attachments;

					logToFile("(Dap_Cron.handleEmailToCsv() - Insert Payload:".$payload2, LOG_INFO_DAP);
					$insert_stmt->bindParam(':key', $key, PDO::PARAM_STR);
					$insert_stmt->bindParam(':payload', $payload2, PDO::PARAM_STR);
					try {
						$insert_stmt->execute();
					} catch (PDOException $e) {
						logToFile($e->getMessage(),LOG_FATAL_DAP);
					}
				}
				fclose($handle);

				//COMPLETED TASK SUCCESSFULLY
				$update_stmt->bindParam(':id', $row['id'], PDO::PARAM_INT);
				$update_stmt->bindValue(':status', 'C', PDO::PARAM_STR);
				$update_stmt->bindValue(':comments', 'Completed Successfully', PDO::PARAM_STR);
				$update_stmt->execute();
			}

			return;
		} catch (PDOException $e) {
			logToFile($e->getMessage(),LOG_FATAL_DAP);
			throw $e;
		} catch (Exception $e) {
			logToFile($e->getMessage(),LOG_FATAL_DAP);
			throw $e;
		}
	}

	/*
	actionType = EMAILTOPRODUCT
	Payload format #PRODUCTID||SUBJECT||BODY||ATTACHMENTS


	*/
	public static function handleEmailToProduct() {
		logToFile("(Dap_Cron.handleEmailToProduct() - Method Init.");
		try {

			$dap_dbh = Dap_Connection::getConnection();
			$select_sql = "select
						id,
						payload,
						actionKey
					from
						dap_mass_actions
					where
						status in ('NEW', 'E') and
						actionType = 'EMAILTOPRODUCT'
					";
			$select_users_sql = "select
						u.id,
						u.first_name,
						u.last_name,
						u.email,
						u.password
					from
						dap_users_products_jn upj,
						dap_users u
					where
						upj.product_id = :product_id and
						upj.status =  'A' and
						u.id = upj.user_id and
						u.status = 'A' and 
						u.opted_out = 'N' and 
						CURDATE() <= upj.access_end_date
					";
			$update_sql = "update
							dap_mass_actions
							set
							status = :status,
							comments = :comments
							where
							id = :id
							";
			$insert_sql = "insert into dap_mass_actions
								(actionType, actionKey, payload, status, priority)
							values ('EMAIL', :key, :payload, 'NEW', 4)
							";

			//logToFile($sql,LOG_DEBUG_DAP);
			$select_stmt = $dap_dbh->prepare($select_sql);
			$select_users_stmt = $dap_dbh->prepare($select_users_sql);
			$update_stmt = $dap_dbh->prepare($update_sql);
			$insert_stmt = $dap_dbh->prepare($insert_sql);

			//execute select
			$select_stmt->execute();

			if ($row = $select_stmt->fetch(PDO::FETCH_ASSOC)) {
				//get payload
				$payload = $row['payload'];
				$parentkey = $row['actionKey'];
				//parse payload
				$tokens = explode("||",$payload);
				if(count($tokens) < 4) {
					$update_stmt->bindParam(':id', $row['id'], PDO::PARAM_INT);
					$update_stmt->bindValue(':status', 'E', PDO::PARAM_STR);
					$update_stmt->bindValue(':comments', 'Payload Format Is Not Correct', PDO::PARAM_STR);
					$update_stmt->execute();
					return;
				}
				//lets get values
				$productId = $tokens[0];
				$subject = $tokens[1];
				$body = $tokens[2];
				$attachments = $tokens[3];
				$dae = Dap_Cron::doesAttachmentsExist($attachments);
				if(!$dae) {
					$update_stmt->bindParam(':id', $row['id'], PDO::PARAM_INT);
					$update_stmt->bindValue(':status', 'E', PDO::PARAM_STR);
					$update_stmt->bindValue(':comments', 'One or more attachments does not exist'.$data, PDO::PARAM_STR);
					$update_stmt->execute();
					// TODO : continue or return ?
					return;
				}
				//see if file exists and read if it does
				logToFile("(Dap_Cron.handleEmailToProduct() - Product Id:".$productId, LOG_INFO_DAP);
				$select_users_stmt->bindParam(':product_id',$productId);
				$select_users_stmt->execute();
				$results2 = $select_users_stmt->fetchAll(PDO::FETCH_ASSOC);
				//lets loops thru all the user ids and create EMAIL transactions
				//while ($user_row = $select_users_stmt->fetch(PDO::FETCH_ASSOC)) {
				foreach ($results2 as $key2 => $user_row) {
				    logToFile("(Dap_Cron.handleEmailToProduct() - User Row Email:".$user_row['email'], LOG_INFO_DAP);
					$payload2 = $user_row['email'] . "||" . $user_row['first_name'] . "||" . $user_row['last_name'];

					$payload2 = $payload2 . "||" . $subject . "||" . $body . "||" . $attachments . "||" . $user_row['password'];
					$key = $parentkey.":".$user_row['email'];
					logToFile("(Dap_Cron.handleEmailToProduct() - Insert Payload:".$payload2, LOG_INFO_DAP);
					$insert_stmt->bindParam(':payload', $payload2, PDO::PARAM_STR);
					$insert_stmt->bindParam(':key', $key, PDO::PARAM_STR);
					try {
						$insert_stmt->execute();
					} catch (PDOException $e) {
						logToFile($e->getMessage(),LOG_FATAL_DAP);
					}
				}

				//COMPLETED TASK SUCCESSFULLY
				$update_stmt->bindParam(':id', $row['id'], PDO::PARAM_INT);
				$update_stmt->bindValue(':status', 'C', PDO::PARAM_STR);
				$update_stmt->bindValue(':comments', 'Completed Successfully', PDO::PARAM_STR);
				$update_stmt->execute();
			}

			return;
		} catch (PDOException $e) {
			logToFile($e->getMessage(),LOG_FATAL_DAP);
			throw $e;
		} catch (Exception $e) {
			logToFile($e->getMessage(),LOG_FATAL_DAP);
			throw $e;
		}
	}

	/*
	actionType = BE-ALLUSERS/BE-ALLAFF/BE-ALLUNCONF//BE-ALLNOPROD
	Payload format SUBJECT||BODY||ATTACHMENTS


	*/
	public static function handleBEAllUsers() {
		logToFile("(Dap_Cron.handleBEAllUsers() - Method Init.");
		try {

			$dap_dbh = Dap_Connection::getConnection();
			$select_sql = "select
						id,
						actionType,
						actionKey,
						payload
					from
						dap_mass_actions
					where
						status in ('NEW', 'E') and
						actionType in ('BE-ALLUSERS', 'BE-ALLAFF', 'BE-ALLUNCONF', 'BE-ALLNOPROD')
					";
			
			
			$select_users_sql_all_users = "select
						u.id,
						u.first_name,
						u.last_name,
						u.email,
						u.password
					from
						dap_users u
					where
						u.status =  'A' and
						u.opted_out = 'N'
					";
			$select_users_sql_all_unconfirmed_users = "select
						u.id,
						u.first_name,
						u.last_name,
						u.email,
						u.password
					from
						dap_users u
					where
						u.status =  'U' and
						u.opted_out = 'N'
					";
			$select_users_sql_all_aff = "select
						u.id,
						u.first_name,
						u.last_name,
						u.email,
						u.password
					from
						dap_users u
					where
						u.status =  'A' and
						u.opted_out = 'N' and
						u.id in (select distinct(affiliate_id) from dap_aff_referrals)
					";
			$select_users_sql_all_noprod = "select
						u.id,
						u.first_name,
						u.last_name,
						u.email,
						u.password
					from 
						dap_users u
					where 
						u.id not in 
						(select distinct user_id from dap_users_products_jn) and
						u.opted_out = 'N'";
	
			
			
			$update_sql = "update
							dap_mass_actions
							set
							status = :status,
							comments = :comments
							where
							id = :id
							";
			
			$insert_sql = "insert into dap_mass_actions
								(actionType, actionKey, payload, status, priority)
							values ('EMAIL', :key, :payload, 'NEW', 4)
							";

			//logToFile($sql,LOG_DEBUG_DAP);
			$select_stmt = $dap_dbh->prepare($select_sql);
			$update_stmt = $dap_dbh->prepare($update_sql);
			$insert_stmt = $dap_dbh->prepare($insert_sql);

			//execute select
			$select_stmt->execute();
			$results = $select_stmt->fetchAll(PDO::FETCH_ASSOC);
			foreach ($results as $key => $row) {
			//while ($row = $select_stmt->fetch(PDO::FETCH_ASSOC)) {
				//get payload
				$payload = $row['payload'];
				$parentkey = $row['actionKey'];
				//parse payload
				$tokens = explode("||",$payload);
				if(count($tokens) < 3) {
					$update_stmt->bindParam(':id', $row['id'], PDO::PARAM_INT);
					$update_stmt->bindValue(':status', 'E', PDO::PARAM_STR);
					$update_stmt->bindValue(':comments', 'Payload Format Is Not Correct', PDO::PARAM_STR);
					$update_stmt->execute();
					return;
				}
				//lets get values
				$subject = $tokens[0];
				$body = $tokens[1];
				$attachments = $tokens[2];
				$dae = Dap_Cron::doesAttachmentsExist($attachments);
				if(!$dae) {
					$update_stmt->bindParam(':id', $row['id'], PDO::PARAM_INT);
					$update_stmt->bindValue(':status', 'E', PDO::PARAM_STR);
					$update_stmt->bindValue(':comments', 'One or more attachments does not exist'.$data, PDO::PARAM_STR);
					$update_stmt->execute();
					// TODO : continue or return ?
					return;
				}
				//see if file exists and read if it does
				if($row['actionType'] == "BE-ALLUSERS") {
					$select_users_stmt = $dap_dbh->prepare($select_users_sql_all_users);
				} else if($row['actionType'] == "BE-ALLAFF") {
					$select_users_stmt = $dap_dbh->prepare($select_users_sql_all_aff);
				} else if($row['actionType'] == "BE-ALLUNCONF") {
					$select_users_stmt = $dap_dbh->prepare($select_users_sql_all_unconfirmed_users);
				}  else if($row['actionType'] == "BE-ALLNOPROD") {
					$select_users_stmt = $dap_dbh->prepare($select_users_sql_all_noprod);
				} else {
					continue;
				}
				
				$select_users_stmt->execute();
				$results2 = $select_users_stmt->fetchAll(PDO::FETCH_ASSOC);
				foreach ($results2 as $key => $user_row) {
				//lets loops thru all the user ids and create EMAIL transactions
				//while ($user_row = $select_users_stmt->fetch(PDO::FETCH_ASSOC)) {
				    logToFile("(Dap_Cron.handleBEAllUsers() - User Row Email:".$user_row['email'], LOG_INFO_DAP);
					$payload2 = $user_row['email'] . "||" . $user_row['first_name'] . "||" . $user_row['last_name'];

					$payload2 = $payload2 . "||" . $subject . "||" . $body . "||" . $attachments . "||" . $user_row['password'];
					$key = $user_row['email'] . ":" . $parentkey;
					logToFile("(Dap_Cron.handleBEAllUsers() - Insert Payload:".$payload2, LOG_INFO_DAP);
					$insert_stmt->bindParam(':payload', $payload2, PDO::PARAM_STR);
					$insert_stmt->bindParam(':key', $key, PDO::PARAM_STR);
					try {
						$insert_stmt->execute();
					} catch (PDOException $e) {
						logToFile($e->getMessage(),LOG_FATAL_DAP);
					} catch (Exception $e) {
						logToFile($e->getMessage(),LOG_FATAL_DAP);
					}
				}

				//COMPLETED TASK SUCCESSFULLY
				$update_stmt->bindParam(':id', $row['id'], PDO::PARAM_INT);
				$update_stmt->bindValue(':status', 'C', PDO::PARAM_STR);
				$update_stmt->bindValue(':comments', 'Completed Successfully', PDO::PARAM_STR);
				$update_stmt->execute();
			}

			return;
		} catch (PDOException $e) {
			logToFile($e->getMessage(),LOG_FATAL_DAP);
			throw $e;
		} catch (Exception $e) {
			logToFile($e->getMessage(),LOG_FATAL_DAP);
			throw $e;
		}
	}


	/*
	actionType=EMAIL
	Payload format
	#TOEMAILID||FIRSTNAME||LASTNAME (optional)||SUBJECT||BODY||COMMADELLISTOFATTACHFILES


	*/
	public static function handleEmail() {
		logToFile("(Dap_Cron.handleEmail() - Method Init.");
		try {
			//initialize the smtpserver config in db.
			Dap_SMTPServer::init();
			//lets go in a large loop
			for($i=0; $i<100;$i++) {
				$sent_email_counter = 0;


			//get smtp server
			$server = Dap_SMTPServer::get();
			if(!isset($server)) {
				logToFile("(Dap_Cron.handleEmail() - SMTPServer Not Available.");
				//break the loop if we dont have any more smtp servers to use;
				return;
			}
			//get batch size for this smtp server
			$batch_size = $server->getUseableLimit();
			//
			$dap_dbh = Dap_Connection::getConnection();
			$select_sql = "select
						id,
						payload
					from
						dap_mass_actions
					where
						status in ('NEW', 'E') and
						actionType = 'EMAIL'
					order by id asc
					limit
					";
			$update_sql = "update
							dap_mass_actions
							set
							status = :status,
							comments = :comments
							where
							id = :id
							";

			//$batch_size = getEmailBatchSize();
			// no more delay
			//$delay = getEmailDelay();
			$select_sql = $select_sql . $batch_size;
			logToFile("(Dap_Cron.handleEmail() - SMTPServer: " . $server->getServer() . "Batch Size: ". $batch_size);
			logToFile("(Dap_Cron.handleEmail() - SQL: $select_sql");

			//logToFile($sql,LOG_DEBUG_DAP);
			$select_stmt = $dap_dbh->prepare($select_sql);
			$update_stmt = $dap_dbh->prepare($update_sql);

			//execute select
			$select_stmt->execute();

			//lets see if we got results. if not, break; return 
			$cols = $select_stmt->columnCount();
			if($cols <= 0) {
				//we dont have any more emails to send from the db, lets break the large loop and return;
				return;
			}
			
			//we have emails to send and we have smtp server handy to send emails.

			//create the mailer object using information from the smtp server config;
			$host = $server->getServer();
			logToFile("SMTP Host To Be Used: $host");
			$mail  = new PHPMailerDAP();
			$mail->SetLanguage('en','language/');
			if("local_web_host" != strtolower($host)) {	
				//$mail->SMTPDebug = true;
				$mail->IsSMTP();
				$mail->SMTPAuth = true;
				$mail->Username = $server->getUserid();
				$mail->Password = $server->getPassword();
				//logToFile("Userid: $mail->Username, Password:$mail->Password,");
				$ssl = $server->getSsl();
				if("y" == strtolower($ssl)) {
					$mail->SMTPSecure = "ssl";
					//$mail->SMTPSecure = "tls";
				}
				$mail->Host = $host;
				$mail->Port = $server->getPort();
			}
			
				
			while ($row = $select_stmt->fetch(PDO::FETCH_ASSOC)) {
				logToFile("(Dap_Cron.handleEmail() - Processing row.");
				//get payload
				$payload = $row['payload'];
				//parse payload
				$tokens = explode("||",$payload);
				if(count($tokens) < 6) {
					logToFile("(Dap_Cron.handleEmail() - Payload Format Is Not Correct", LOG_FATAL_DAP);
					$update_stmt->bindParam(':id', $row['id'], PDO::PARAM_INT);
					$update_stmt->bindValue(':status', 'E', PDO::PARAM_STR);
					$update_stmt->bindValue(':comments', 'Payload Format Is Not Correct', PDO::PARAM_STR);
					$update_stmt->execute();
					continue;
				}
				//lets get data
				$to = $tokens[0];
				$first_name = $tokens[1];
				$last_name = $tokens[2];
				$subject = $tokens[3];
				if(isset($tokens[6]) && $tokens[6] != "") {
					$password = $tokens[6];
				}
				$subject = personalizeMessageDet($to, $first_name, $last_name, $subject, $password); //str_replace("%%FIRST_NAME%%", $first_name, $subject);
				//TODO: transform the subject
				$body = $tokens[4];
				//$body = personalizeMessageDet($to, $first_name, $last_name, $body, $password); //str_replace("%%FIRST_NAME%%", $first_name, $subject);
				$body = $body . "\n\n\n\n" . getEmailFooter();
				$body = personalizeMessage(Dap_User::loadUserByEmail($to), $body); 
				//TODO: Transform the body
				$attach_str = $tokens[5];
				if(!isset($first_name) || $first_name == "") {
					logToFile("(Dap_Cron.handleEmail() - Missing First Name", LOG_ERROR_DAP);
					$update_stmt->bindParam(':id', $row['id'], PDO::PARAM_INT);
					$update_stmt->bindValue(':status', 'E', PDO::PARAM_STR);
					$update_stmt->bindValue(':comments', 'Payload Format Is Not Correct - Missing First Name', PDO::PARAM_STR);
					$update_stmt->execute();
					continue;
				}
				//
				if(!isset($subject) || $subject == "") {
					logToFile("(Dap_Cron.handleEmail() - Missing Subject", LOG_ERROR_DAP);
					$update_stmt->bindParam(':id', $row['id'], PDO::PARAM_INT);
					$update_stmt->bindValue(':status', 'E', PDO::PARAM_STR);
					$update_stmt->bindValue(':comments', 'Payload Format Is Not Correct - Missing Subject', PDO::PARAM_STR);
					$update_stmt->execute();
					continue;
				}
				//
				if(!isset($body) || $body == "") {
					logToFile("(Dap_Cron.handleEmail() - Missing Body", LOG_ERROR_DAP);
					$update_stmt->bindParam(':id', $row['id'], PDO::PARAM_INT);
					$update_stmt->bindValue(':status', 'E', PDO::PARAM_STR);
					$update_stmt->bindValue(':comments', 'Payload Format Is Not Correct - Missing Body', PDO::PARAM_STR);
					$update_stmt->execute();
					continue;
				}
				logToFile("(Dap_Cron.handleEmail() - Attachment str:*".$attach_str."*");
				//lets get list of attachments
				$from = getAdminEmail();
				$admin_name = Dap_Config::get("ADMIN_NAME");
				//attach footer
				//$body = $body . "\n\n\n\n" . getEmailFooter();
				//set From
				//$body = str_replace("\\", '', $body);
				
				//clear any previous stuff.
				$mail->ClearAllRecipients();
				$mail->ClearAttachments();
				//set new stuff
				$mail->From = $from;
				$mail->FromName = $admin_name;
				$mail->AddAddress($to);
				$mail->Subject = $subject;
				//$mail->MsgHTML($body);
				$pieces = explode(Dap_Config::get('HTMLSEPARATOR'),$body);
				//logToFile("pieces[0]: $pieces[0]"); 
				//logToFile("pieces[1]: $pieces[1]"); 
				$textBody = ($pieces[0] == "") ? "Sorry, this email is only being sent in HTML format." : $pieces[0];
				$htmlBody = $pieces[1];
				$mail->Body = $textBody;
				if($htmlBody != "") {
					$mail->Body = $htmlBody;
					$mail->isHTML = true;
					$mail->AltBody = $textBody;
				}


				if(isset($attach_str) && trim($attach_str) != "")  {
					logToFile("(Dap_Cron.handleEmail() - Attachment str is not empty:*".$attach_str."*");
					//lets process attachments and make sure they are all readable and exists. If not, write error and continue with processing other EMAILs.
					$attachs = explode(",",$attach_str);
					//for each attachment
					foreach($attachs as $filename) {
						$filename = DAP_ROOT . '/admin/attachments/' . $filename;
						if(is_readable($filename)) {
						    $mail->AddAttachment($filename);
						} else {
							//mark this as error and move on
							logToFile("(Dap_Cron.handleEmail() - Attachment does not exists:*".$filename."*");
							$update_stmt->bindParam(':id', $row['id'], PDO::PARAM_INT);
							$update_stmt->bindValue(':status', 'E', PDO::PARAM_STR);
							$update_stmt->bindValue(':comments', 'Attachment does not exist', PDO::PARAM_STR);
							$update_stmt->execute();
							continue 2;
						}
					}
				}

				// now we just send the message
				//if (@mail($to, $subject, $message, $headers)) {
				logToFile("Here, about to send...");
				if (!$mail->Send()) {
					logToFile("(Dap_Cron.handleEmail() - Message Could NOT Be Sent To:".$to."*".$mail->ErrorInfo, LOG_ERROR_DAP);
					$update_stmt->bindParam(':id', $row['id'], PDO::PARAM_INT);
					$update_stmt->bindValue(':status', 'E', PDO::PARAM_STR);
					$update_stmt->bindValue(':comments', 'Failed.'.$mail->ErrorInfo, PDO::PARAM_STR);
					$update_stmt->execute();
				} else {
					//echo "Message Sent";
					logToFile("(Dap_Cron.handleEmail() - Message Sent To:".$to."*");
					//COMPLETED TASK SUCCESSFULLY
					$update_stmt->bindParam(':id', $row['id'], PDO::PARAM_INT);
					$update_stmt->bindValue(':status', 'C', PDO::PARAM_STR);
					$update_stmt->bindValue(':comments', 'Completed Successfully', PDO::PARAM_STR);
					$update_stmt->execute();
					//up the sent email count
					$sent_email_counter++;
					//lets sleep a bit, if the message was sent successfully.
					//logToFile("(Dap_Cron.handleEmail() - Sleeping (Delay) For:".$delay."*", LOG_INFO_DAP);
					//sleep($delay);
				}
			}

			//we need to go and update the stats in SMTP Server count
			logToFile("(Dap_Cron.handleEmail() - Sent Email Counts:".$sent_email_counter."*");
			if(isset($server) && $sent_email_counter > 0) {
				$server->updateRunningTotal($sent_email_counter);
			}
			//end of large outer loop;
			}
			return;
		} catch (PDOException $e) {
			logToFile($e->getMessage(),LOG_FATAL_DAP);
			throw $e;
		} catch (Exception $e) {
			logToFile($e->getMessage(),LOG_FATAL_DAP);
			throw $e;
		}
	}
	/*
	* Cron for paying affiliates for sale
	- Take all transactions in certain finished state (user product created) ---- //Take all transactions from dap trans table that are  NOT in aff_transactions.
	- Get userid and product id for each of these trans using payer email and product id.
	- Get affiliate id from aff_referrals for the given userid, product id.
	- Check aff_transactions to see if this product sale commission is already paid and its not recurring pay enabled. Exit if its not recurring
		and already sale paid. Mark the trans as already aff processed.
		//TODO: HOW TO MARK THE TRANS AS ALREADY PROCESSED IN THIS CASE ?
	- Check commission structure from aff_comm to see if this product id, affiliateId has per sale commission.
	- If yes, calculate sale commission from trans payment_value and type of sale commision (%age or fixed) and insert into
		aff_transactions with this transid and aff_referrals_id.
	- Mark the transaction as aff processed.
	*/

	public static function processTransactionsForAffiliation() {
		logToFile("(Dap_Cron.processTransactionsForAffiliation() - Method Init.");
		try {
			$dap_dbh = Dap_Connection::getConnection();
			$trans_select_sql = "select *
					from dap_transactions
					where
					status = 5 and
					trans_type != 'subscr_signup'
					and (date between (now() - interval 7 day) and now() )
					";
			$referrals_select_sql = "select id, affiliate_id
					from dap_aff_referrals
					where
					user_id = :user_id and
					product_id = :product_id
					";
			$aff_trans_select_sql = "select *
					from dap_aff_earnings
					where
					aff_referrals_id = :aff_referrals_id and
					earning_type = 'S'
					";
			$aff_trans_insert_sql = "insert into dap_aff_earnings
					(aff_referrals_id, amount_earned, datetime, transaction_id, earning_type)
					values
					(:aff_referrals_id, :amount_earned, now(), :transaction_id, 'S')
					";
			//echo "sql: $sql<br>"; exit;
			logToFile("processTransactionsForAffiliations:SQL:".$trans_select_sql);
			$trans_select_stmt = $dap_dbh->prepare($trans_select_sql);
			$referrals_select_stmt = null;
			$aff_trans_select_stmt = null;
			$aff_trans_insert_stmt = null;
			$trans_select_stmt->execute();

			//Statements to build the links
			while ($trans_select_row = $trans_select_stmt->fetch(PDO::FETCH_ASSOC)) {
				
				try {
					$referrals_select_stmt = $dap_dbh->prepare($referrals_select_sql);
					$aff_trans_select_stmt = $dap_dbh->prepare($aff_trans_select_sql);
					$aff_trans_insert_stmt = $dap_dbh->prepare($aff_trans_insert_sql);
					
					//get userid and product id
					$user = Dap_User::loadUserByEmail($trans_select_row['payer_email']);
					if(!isset($user)) continue;
					$processAffiliation = false;
					$affiliateId = null;
					$aff_ref_id = null;
					
					$userId = $user->getId();
					$productId = $trans_select_row['product_id'];
					$transId = $trans_select_row['id'];
					$transValue = $trans_select_row['payment_value'];
					logToFile("Dap_Cron.processTransactionsForAffiliation() - Processing TransactionId:".$transId.", UserId:".$userId.", ProductId:".$productId);
					
					
					//lets see if we got some referral id for this user/product
					$referrals_select_stmt->bindParam(':user_id', $userId, PDO::PARAM_INT);
					$referrals_select_stmt->bindParam(':product_id', $productId, PDO::PARAM_INT);
					$referrals_select_stmt->execute();
					
					logToFile("Dap_Cron.processTransactionsForAffiliation() - Past fail block for transId:".$transId.", UserId:".$userId.", ProductId:".$productId);
					$referrals_select_row = $referrals_select_stmt->fetch(PDO::FETCH_ASSOC);
					
					if( !isset($referrals_select_row) || ($referrals_select_row == null) ) {
						//No affiliate referral found - could have referral for a different product purchased previously
						logToFile("User id: $userId - no referral found for product $productId");
						
						//Check if affiliate exists for this user
						$affiliateId = $user->getAffiliate();
						if(isset($affiliateId)) {
							//Affiliate exists
							$processAffiliation = true;
							logToFile("Found old affiliateId: $affiliateId");
							
							//Stamp pending products of user with affiliate id
							if( (Dap_Config::get("ALLOW_SELF_REFERRAL") == "Y") || ($affiliateId != $userId) ) {
								logToFile("Figure out all products for user $userId for which no affiliate id has been set"); 
								//1. Figure out all products for this user, for which no affiliate id has been set
								$ProductListArray = Dap_AffReferrals::getProductsPendingAffiliateStamping($userId);
								
								if( sizeof($ProductListArray != 0) ){
									//Process Affiliate Referral Insertion & Process Lead
									logToFile("Processing affiliations for unstamped products for user $userId and affiliate $affiliateId"); 
									Dap_AffReferrals::processAffiliation($affiliateId, $userId, $ProductListArray);
								}
							}
						} 
						continue;
					} 
					else if($referrals_select_row) {
						//we have a referral
						$affiliateId = $referrals_select_row['affiliate_id'];
						$aff_ref_id = $referrals_select_row['id'];
						$processAffiliation = true;
						logToFile("Dap_Cron.processTransactionsForAffiliation() - We have referrer for the user/product. Affiliate Id:".$affiliateId, LOG_INFO_DAP);
					}
					
					//If either new affiliate, or old aff existed, process...
					if($processAffiliation) {
						//load dap_aff_comm (commision setup) data to see if this has any commission setup and is reccurring.
						//we have to pass affiliateId and not the userid of the person who bought th eproduct
						$dap_aff_comm = Dap_AffCommissions::load($affiliateId, $productId);
						if($dap_aff_comm == NULL) {
							//no commission setup, skip this transaction.
							Dap_Transactions::setRecordStatus($transId, 7);
							logToFile("Dap_Cron.processTransactionsForAffiliation() - NO COMMISSION SETUP. Marking Transaction as complete..TransId:".$transId,info
							);
							continue;
						}
						//check if this is sale commission setup
						if($dap_aff_comm->isSaleCommission() === FALSE)  {
							//no commission setup, skip this transaction.
							Dap_Transactions::setRecordStatus($transId, 7);
							logToFile("Dap_Cron.processTransactionsForAffiliation() - NO SALE COMMISSION SETUP. Marking Transaction as complete..TransId:".$transId, LOG_INFO_DAP);
							continue;
						}
						//we have commission setup.
						if($dap_aff_comm->getIs_comm_recurring() == "N") {
							//lets check if we already paid this guy for sale and this is not recurring commissions. If yes, mark trans complete and continue.
							$aff_trans_select_stmt->bindParam(':aff_referrals_id', $aff_ref_id, PDO::PARAM_INT);
							$aff_trans_select_stmt->execute();
							if ($aff_trans_select_row = $aff_trans_select_stmt->fetch(PDO::FETCH_ASSOC)) {
								//we already credited this guy before and this is not recurring. So mark trans and move on.
								Dap_Transactions::setRecordStatus($transId, 7);
								logToFile("Dap_Cron.processTransactionsForAffiliation() - WE ALREADY CREDITED THIS AFFILIATE AND ITS NOT RECURRING COMMISSION. Marking Transaction as complete..TransId:".$transId, LOG_INFO_DAP);
								continue;
							}
						}
						//We are here because either this is recurring commission and/or we did not credit this affiliate before.
						$sale_commission = $dap_aff_comm->calculateSaleCommission($transValue);
						logToFile("Dap_Cron.processTransactionsForAffiliation() - Calculated Sale Commission:".$sale_commission, LOG_INFO_DAP);
						$aff_trans_insert_stmt->bindParam(':aff_referrals_id', $aff_ref_id, PDO::PARAM_INT);
						$aff_trans_insert_stmt->bindParam(':amount_earned', $sale_commission, PDO::PARAM_INT);
						$aff_trans_insert_stmt->bindParam(':transaction_id', $transId, PDO::PARAM_INT);
						$aff_trans_insert_stmt->execute();
						Dap_Transactions::setRecordStatus($transId, 7);
						logToFile("Dap_Cron.processTransactionsForAffiliation() - Commission Credited for the amount: $sale_commission . Marking Transaction as complete..TransId:".$transId, LOG_INFO_DAP);
						
						
						//Do N-tier processing
						//processAffiliateCommissionsRecursively($aff_ref_id, $userId, $affiliateId, $productId, $transId, $transValue, 2);
						Dap_AffCommissions::processAffiliateCommissionsMultiTierRecursive($userId, $affiliateId, $productId, $transId, $transValue, 2);
	
					} else {
						logToFile("Dap_Cron.processTransactionsForAffiliation() - We dont have referrer for the user/product.");
						continue;
					}
					
					$referrals_select_stmt = null;
					$referrals_select_row = null;
				
				} //end try inside while
				catch (PDOException $e) {
					logToFile($e->getMessage(),LOG_FATAL_DAP);
					throw $e;
				} catch (Exception $e) {
					logToFile($e->getMessage(),LOG_FATAL_DAP);
					throw $e;
				}
				
			} //end while
			
			$trans_select_stmt = null;
			$referrals_select_stmt = null;
			$aff_trans_select_stmt = null;
			$aff_trans_insert_stmt = null;
			$dap_dbh = null;
			
		} catch (PDOException $e) {
			logToFile($e->getMessage(),LOG_FATAL_DAP);
			throw $e;
		} catch (Exception $e) {
			logToFile($e->getMessage(),LOG_FATAL_DAP);
			throw $e;
		}

	}


	/*
	actionType =  BULKPAYAFFTOCSV
	Payload format CSVFILENAME||COMMENTS

	*/
	public static function handleBulkPayAffiliates() {
		logToFile("(Dap_Cron.handleBulkPayAffiliates() - Method Init.");
		try {

			$dap_dbh = Dap_Connection::getConnection();
			$select_sql = "select
						id,
						payload,
						actionKey
					from
						dap_mass_actions
					where
						status in ('NEW', 'E') and
						actionType = 'BULKPAYAFFTOCSV'
					";
			$update_sql = "update
							dap_mass_actions
							set
							status = :status,
							comments = :comments
							where
							id = :id
							";
			$insert_sql = "insert into dap_mass_actions
								(actionType, actionKey, payload, status)
							values ('PAYAFFILIATE', :key, :payload, 'NEW')
							";

			//logToFile($sql,LOG_DEBUG_DAP);
			$select_stmt = $dap_dbh->prepare($select_sql);
			$update_stmt = $dap_dbh->prepare($update_sql);
			$insert_stmt = $dap_dbh->prepare($insert_sql);

			//execute select
			$select_stmt->execute();

			if ($row = $select_stmt->fetch(PDO::FETCH_ASSOC)) {
				//get payload
				$payload = $row['payload'];
				$parentkey = $row['actionKey'];
				//parse payload
				$tokens = explode("||",$payload);
				if(count($tokens) < 1) {
					$update_stmt->bindParam(':id', $row['id'], PDO::PARAM_INT);
					$update_stmt->bindValue(':status', 'E', PDO::PARAM_STR);
					$update_stmt->bindValue(':comments', 'Payload Format Is Not Correct', PDO::PARAM_STR);
					$update_stmt->execute();
					return;
				}
				//lets get values
				$csvfile = $tokens[0];
				//TODO: confirm if comments are at file level or individual payment level
				if(isset($tokens[1])) {
					$comment = $tokens[1];
				} else {
					$comment = "Bulk Payments";
				}
				//see if file exists and read if it does
				if(!file_exists($csvfile)) {
					logToFile("(Dap_Cron.handleBulkPayAffiliates() - File Doesnt Exist:".$csvfile,LOG_ERROR_DAP);
					$update_stmt->bindParam(':id', $row['id'], PDO::PARAM_INT);
					$update_stmt->bindValue(':status', 'E', PDO::PARAM_STR);
					$update_stmt->bindValue(':comments', 'CSV File Does not exist', PDO::PARAM_STR);
					$update_stmt->execute();
					return;
				}
				logToFile("(Dap_Cron.handleBulkPayAffiliates() - Opening CSV File:".$csvfile);
				$handle = fopen($csvfile, "r");
				if($handle === FALSE) {
					logToFile("(Dap_Cron.handleBulkPayAffiliates() - Error on file open".$csvfile, LOG_ERROR_DAP);
					$update_stmt->bindParam(':id', $row['id'], PDO::PARAM_INT);
					$update_stmt->bindValue(':status', 'E', PDO::PARAM_STR);
					$update_stmt->bindValue(':comments', 'Error opening CSV file', PDO::PARAM_STR);
					$update_stmt->execute();
					return;
				}
				logToFile("(Dap_Cron.handleBulkPayAffiliates() - Opening CSV File:".$handle);
				while (($data = fgetcsv($handle, 1000, "\t")) !== FALSE) {
					logToFile("(Dap_Cron.handleBulkPayAffiliates() - CSV Row:".$data);
					if(count($data) < 2) {
						$update_stmt->bindParam(':id', $row['id'], PDO::PARAM_INT);
						$update_stmt->bindValue(':status', 'E', PDO::PARAM_STR);
						$update_stmt->bindValue(':comments', 'Error in CSV File'.$data, PDO::PARAM_STR);
						$update_stmt->execute();
						return;
					}
					//data[0] is email id
					//data[1] is amount
					//then comments
					$payload2 = $data[0] . "||" . $data[1] . "||" . $comment;
					$key = $data[0] . ":" . $parentkey;
					logToFile("(Dap_Cron.handleBulkPayAffiliates() - Insert Payload:".$payload2, LOG_INFO_DAP);
					$insert_stmt->bindParam(':key', $key, PDO::PARAM_STR);
					$insert_stmt->bindParam(':payload', $payload2, PDO::PARAM_STR);
					$insert_stmt->execute();
				}
				fclose($handle);
				
				//COMPLETED TASK SUCCESSFULLY
				$update_stmt->bindParam(':id', $row['id'], PDO::PARAM_INT);
				$update_stmt->bindValue(':status', 'C', PDO::PARAM_STR);
				$update_stmt->bindValue(':comments', 'Completed Successfully', PDO::PARAM_STR);
				$update_stmt->execute();
				//Delete the affiliate csv file now.
				unlink($csvfile);	
			}

			return;
		} catch (PDOException $e) {
			logToFile($e->getMessage(),LOG_FATAL_DAP);
			throw $e;
		} catch (Exception $e) {
			logToFile($e->getMessage(),LOG_FATAL_DAP);
			throw $e;
		}
	}
	/*
	actionType=PAYAFFILIATE
	Payload format
	EMAIL||AMOUNT||COMMENT
	*/
	public static function handlePayAffiliate() {
		logToFile("(Dap_Cron.handlePayAffiliate() - Method Init.");
		try {

			$dap_dbh = Dap_Connection::getConnection();
			$select_sql = "select
						id,
						payload,
						actionKey
					from
						dap_mass_actions
					where
						status in ('NEW', 'E') and
						actionType = 'PAYAFFILIATE'
					";
			$update_sql = "update
							dap_mass_actions
							set
							status = :status,
							comments = :comments
							where
							id = :id
							";
			$insert_sql = "insert into dap_mass_actions
								(actionType, actionKey, payload, status, priority)
							values ('EMAIL', :key, :payload, 'NEW', 4)
							";

			//logToFile($sql,LOG_DEBUG_DAP);
			$select_stmt = $dap_dbh->prepare($select_sql);
			$update_stmt = $dap_dbh->prepare($update_sql);
			$insert_stmt = $dap_dbh->prepare($insert_sql);

			//execute select
			$select_stmt->execute();

			while ($row = $select_stmt->fetch(PDO::FETCH_ASSOC)) {
				logToFile("(Dap_Cron.handlePayAffiliate() - Processing row.");
				//get payload
				$payload = $row['payload'];
				$parentkey = $row['actionKey'];
				//parse payload
				$tokens = explode("||",$payload);
				if(count($tokens) < 3) {
					logToFile("(Dap_Cron.handlePayAffiliate() - Payload Format Is Not Correct", LOG_ERROR_DAP);
					$update_stmt->bindParam(':id', $row['id'], PDO::PARAM_INT);
					$update_stmt->bindValue(':status', 'E', PDO::PARAM_STR);
					$update_stmt->bindValue(':comments', 'Payload Format Is Not Correct', PDO::PARAM_STR);
					$update_stmt->execute();
					continue;
				}
				//lets get data
				$user = Dap_User::loadUserByEmail($tokens[0]);
				if(!isset($user)) {
					logToFile("(Dap_Cron.handlePayAffiliate() - Affiliate Doesn't Exist For User", LOG_ERROR_DAP);
					$update_stmt->bindParam(':id', $row['id'], PDO::PARAM_INT);
					$update_stmt->bindValue(':status', 'E', PDO::PARAM_STR);
					$update_stmt->bindValue(':comments', 'Affiliate Does Not Exist For User', PDO::PARAM_STR);
					$update_stmt->execute();
					continue;

				}
				$userId = $user->getId();
				$to = $tokens[0];
				$amount = $tokens[1];
				$comment = $tokens[2];
				$first_name = $user->getFirst_name();
				$last_name = $user->getLast_name();
				//insert aff payment
				$affpayment = new Dap_AffPayments();
				$affpayment->setAffiliate_id($userId);
				$affpayment->setAmount_paid($amount);
				$affpayment->setComments($comment);
				$affpayment->create();
				//insert done
				//TODO - get subject, body from config
				$subject = Dap_Config::get("AFF_PAYMENT_EMAIL_SUBJECT"); //"Hi %%FIRST_NAME%%, You Got Affiliate Payment";
				$subject = personalizeMessage($user, $subject); //str_replace("%%FIRST_NAME%%", $first_name, $subject);
				//TODO
				$body = Dap_Config::get("AFF_PAYMENT_EMAIL_CONTENT"); //"Hello %%FIRST_NAME%% %%LAST_NAME%%, You got affliate payment of %%AMOUNT%%";
				$body = personalizeMessage($user, $body); //str_replace("%%FIRST_NAME%%", $first_name, $subject);
				$body = str_replace("%%AMOUNT%%", $amount, $body);
				//DO NOT PUT FOOTER HERE. It will be done in handle Email job
				//$body = $body . "\n\n\n\n" . getEmailFooter();
				//insert into the mass action email table. 
				$payload2 = $to . "||" . $first_name . "||" . $last_name;
				$payload2 = $payload2 . "||" . $subject . "||" . $body . "||" . $attachments . "||" ;
				$key = $user_row['email'] . ":" . $parentkey;
				logToFile("(Dap_Cron.handlePayAffiliates() - Insert Payload:".$payload2, LOG_INFO_DAP);
				$insert_stmt->bindParam(':payload', $payload2, PDO::PARAM_STR);
				$insert_stmt->bindParam(':key', $key, PDO::PARAM_STR);
				try {
					$insert_stmt->execute();
					$update_stmt->bindParam(':id', $row['id'], PDO::PARAM_INT);
					$update_stmt->bindValue(':status', 'C', PDO::PARAM_STR);
					$update_stmt->bindValue(':comments', 'Completed Successfully', PDO::PARAM_STR);
					$update_stmt->execute();
					continue;
				} catch (PDOException $e) {
					logToFile($e->getMessage(),LOG_FATAL_DAP);
				} catch (Exception $e) {
					logToFile($e->getMessage(),LOG_FATAL_DAP);
				}
				logToFile("(Dap_Cron.handlePayAffiliates() - Processing Failed:".$payload, LOG_INFO_DAP);
				$update_stmt->bindParam(':id', $row['id'], PDO::PARAM_INT);
				$update_stmt->bindValue(':status', 'E', PDO::PARAM_STR);
				$update_stmt->bindValue(':comments', 'Processing Failed...', PDO::PARAM_STR);
				$update_stmt->execute();
			}

			return;
		} catch (PDOException $e) {
			logToFile($e->getMessage(),LOG_FATAL_DAP);
			throw $e;
		} catch (Exception $e) {
			logToFile($e->getMessage(),LOG_FATAL_DAP);
			throw $e;
		}
	}
	/*
		Identify users and their email resources to be sent today and add them to mass actions table with action type EMAIL
	*/
	public static function sendEmailResources() {
		logToFile("(Dap_Cron.sendEmailResources() - Method Init.");
		try {

			$dap_dbh = Dap_Connection::getConnection();
			$select_sql = "
				SELECT 
					u.email, 
					u.first_name, 
					u.last_name, 
					u.password, 
					er.id, 
					er.subject, 
					er.message, 
					er.attachment, 
					er.sendTo3rdParty, 
					er.thirdPartyEmailId, 
					curdate() as date,
					upj.transaction_id as transid, 
					prj.is_free
				FROM
					dap_products p,
					dap_products_resources_jn prj,
					dap_email_resources er,
					dap_users u,
					dap_users_products_jn upj
				WHERE
					u.status = 'A'
					and u.opted_out = 'N'
					and prj.resource_id = er.id
					and prj.status = 'A'
					and prj.resource_type = 'E'
					and p.id = prj.product_id
					and p.status = 'A'
					and upj.user_id = u.id
					and upj.product_id = p.id
					and upj.status = 'A'
					and curdate() between upj.access_start_date and upj.access_end_date
					and ( datediff(curdate(), upj.access_start_date) = prj.start_day - 1  or
						prj.start_date = curdate() )
					";
			$insert_sql = "insert into dap_mass_actions
								(actionType, actionKey, payload, status, priority)
							values ('EMAIL', :key, :payload, 'NEW', 2)
							";

			//logToFile($sql,LOG_DEBUG_DAP);
			$select_stmt = $dap_dbh->prepare($select_sql);
			$insert_stmt = $dap_dbh->prepare($insert_sql);

			//execute select
			$select_stmt->execute();
			$results = $select_stmt->fetchAll(PDO::FETCH_ASSOC);
			//while ($row = $select_stmt->fetch(PDO::FETCH_ASSOC)) {
			foreach ($results as $key => $row) {
				try {
					logToFile("(Dap_Cron.sendEmailResources() - Processing row.");
					if((($row["transid"] == "-2") || ($row["transid"] == "-1")) && (strtolower($row["is_free"]) != "y")) {
						//user is direct signup/admin not paid but resource is not free, skip this row
						logToFile("(Dap_Cron.sendEmailResources() - Email Resource is not free but user is not paid, skipping. Resource Subject: ".$row['subject'], LOG_INFO_DAP);
						continue;
					}
					$email = $row['email'];
					$firstname = $row['first_name'];
					$lastname = $row['last_name'];
					$subject = $row['subject'];
					$body = $row['message'];
					$attachment = $row['attachment'];
					$password = $row['password'];
					$sendTo3rdParty = $row['sendTo3rdParty'];
					$thirdPartyEmailId = $row['thirdPartyEmailId'];
					$key = "";
					
					//TODO transfer attachment into proper format
					
					if($sendTo3rdParty == "Y") {					
						$body = personalizeMessage(Dap_User::loadUserByEmail($email), $body); //first personalize with user's email id
						$subject = personalizeMessage(Dap_User::loadUserByEmail($email), $subject); //first personalize with user's email id
						$key = $row['id'] . ":" . $thirdPartyEmailId . "-" . $row['email'] . ":ER:" . $row['date'];
						$email = $thirdPartyEmailId; //Switch primary recipient to thirdPartyEmailId instead of actual user
					} else {
						$key = $row['id'] . ":" . $email . ":ER:" . $row['date'];
					}
					
					$payload = $email . "||" . $firstname . "||" . $lastname . "||" . $subject . "||" . $body . "||" . $attachment . "||" . $password;
					
					logToFile("(Dap_Cron.sendEmailResources() - Insert Payload:".$payload, LOG_INFO_DAP);
					$insert_stmt->bindParam(':payload', $payload, PDO::PARAM_STR);
					$insert_stmt->bindParam(':key', $key, PDO::PARAM_STR);
					$insert_stmt->execute();
				} catch (PDOException $e) {
					if( stristr($e->getMessage(), 'Integrity constraint violation') === FALSE ) {
						logToFile($e->getMessage(),LOG_FATAL_DAP);
					}
				} catch (Exception $e) {
					logToFile($e->getMessage(),LOG_FATAL_DAP);
				}
			}

			return;
		} catch (PDOException $e) {
			logToFile($e->getMessage(),LOG_FATAL_DAP);
			throw $e;
		} catch (Exception $e) {
			logToFile($e->getMessage(),LOG_FATAL_DAP);
			throw $e;
		}
	}

	public static function doesAttachmentsExist($attach_str) {
		if(isset($attach_str) && trim($attach_str) != "")  {
			logToFile("(Dap_Cron.doesAttachmentsExist() - Attachment str is not empty:*".$attach_str."*");
			//lets process attachments and make sure they are all readable and exists. If not, write error and continue with processing other EMAILs.
			$attachs = explode(",",$attach_str);
			//for each attachment
			foreach($attachs as $filename) {
				$filename = DAP_ROOT . '/admin/attachments/' . $filename;
				if(is_readable($filename)) {
					//do nothing
				} else {
					logToFile("(Dap_Cron.doesAttachmentExist() - Attachment does not exists:*".$filename."*");
					return false;
				}
			}

		}
		return true;
	}
	
	
	
/*
	actionType = BE-QUERY
	Payload format BE-QUERY||SUBJECT||BODY||ATTACHMENTS


	*/
	public static function handleBEQuery() {
		logToFile("(Dap_Cron.handleBEQuery() - Method Init.");
		try {

			$dap_dbh = Dap_Connection::getConnection();
			$select_sql = "select
						id,
						actionType,
						actionKey,
						payload
					from
						dap_mass_actions
					where
						status in ('NEW', 'E') and
						actionType = 'BE-QUERY'
					";
			
			$update_sql = "update
							dap_mass_actions
							set
							status = :status,
							comments = :comments
							where
							id = :id
							";
			
			$insert_sql = "insert into dap_mass_actions
								(actionType, actionKey, payload, status, priority)
							values ('EMAIL', :key, :payload, 'NEW', 4)
							";

			//logToFile($sql,LOG_DEBUG_DAP);
			$select_stmt = $dap_dbh->prepare($select_sql);
			$update_stmt = $dap_dbh->prepare($update_sql);
			$insert_stmt = $dap_dbh->prepare($insert_sql);

			//execute select
			$select_stmt->execute();
			$results = $select_stmt->fetchAll(PDO::FETCH_ASSOC);
			foreach ($results as $key => $row) {
				//get payload
				$payload = $row['payload'];
				$parentkey = $row['actionKey'];
				//parse payload
				$tokens = explode("||",$payload);
				//Update row if error in payload
				if(count($tokens) < 4) {
					$update_stmt->bindParam(':id', $row['id'], PDO::PARAM_INT);
					$update_stmt->bindValue(':status', 'E', PDO::PARAM_STR);
					$update_stmt->bindValue(':comments', 'Payload Format Is Not Correct', PDO::PARAM_STR);
					$update_stmt->execute();
					return;
				}
				//No error in payload - so continue
				//Get values first
				$sql = $tokens[0];
				$subject = $tokens[1];
				$body = $tokens[2];
				$attachments = $tokens[3];
				$dae = Dap_Cron::doesAttachmentsExist($attachments);
				if(!$dae) {//attachment variable has data, but no such file found
					$update_stmt->bindParam(':id', $row['id'], PDO::PARAM_INT);
					$update_stmt->bindValue(':status', 'E', PDO::PARAM_STR);
					$update_stmt->bindValue(':comments', 'One or more attachments does not exist'.$data, PDO::PARAM_STR);
					$update_stmt->execute();
					// TODO : continue or return ?
					return;
				}
				//Process query
				$select_users_stmt = $dap_dbh->prepare($sql);
				$select_users_stmt->execute();
				$results2 = $select_users_stmt->fetchAll(PDO::FETCH_ASSOC);
				foreach ($results2 as $key => $user_row) {
					//lets loop thru all the user ids and create EMAIL transactions
				    logToFile("(Dap_Cron.handleBEQuery() - User Row Email:".$user_row['email'], LOG_INFO_DAP);
					$payload2 = $user_row['email'] . "||" . $user_row['first_name'] . "||" . $user_row['last_name'];

					$payload2 = $payload2 . "||" . $subject . "||" . $body . "||" . $attachments . "||" . $user_row['password'];
					$key = $user_row['email'] . ":" . $parentkey;
					logToFile("(Dap_Cron.handleBEQuery() - Insert Payload:".$payload2, LOG_INFO_DAP);
					$insert_stmt->bindParam(':payload', $payload2, PDO::PARAM_STR);
					$insert_stmt->bindParam(':key', $key, PDO::PARAM_STR);
					try {
						$insert_stmt->execute();
					} catch (PDOException $e) {
						logToFile($e->getMessage(),LOG_FATAL_DAP);
					} catch (Exception $e) {
						logToFile($e->getMessage(),LOG_FATAL_DAP);
					}
				}

				//COMPLETED TASK SUCCESSFULLY
				$update_stmt->bindParam(':id', $row['id'], PDO::PARAM_INT);
				$update_stmt->bindValue(':status', 'C', PDO::PARAM_STR);
				$update_stmt->bindValue(':comments', 'Completed Successfully', PDO::PARAM_STR);
				$update_stmt->execute();
			}

			return;
		} catch (PDOException $e) {
			logToFile($e->getMessage(),LOG_FATAL_DAP);
			throw $e;
		} catch (Exception $e) {
			logToFile($e->getMessage(),LOG_FATAL_DAP);
			throw $e;
		}
	}
	
/*
SELECT u.email, er.subject, er.message, er.attachment

FROM
	dap_products p,
	dap_products_resources_jn prj,
	dap_email_resources er,
	dap_users u,
	dap_users_products_jn upj
WHERE
	u.status = 'A'
	and prj.resource_id = er.id
 	and prj.status = 'A'
	and prj.resource_type = 'E'
	and p.id = prj.product_id
	and p.status = 'A'
	and upj.user_id = u.id
	and upj.product_id = p.id
	and upj.status = 'A'
	and curdate() between upj.access_start_date and upj.access_end_date
	-- and difference (in days) between today and start date is equal to prj.start_day
	and datediff(curdate(), upj.access_start_date) = prj.start_day
*/

}
?>
